<?php

namespace Drupal\jstimer\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides the configuration form for JSTimer settings.
 *
 * This class handles the form where the admin can configure the settings
 * related to the JSTimer module, including clock types and related options.
 *
 * @see \Drupal\Core\Form\ConfigFormBase
 */
class JstimerAdminSettings extends ConfigFormBase {

  /**
   * The module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The configuration factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs a JstimerAdminSettings object.
   *
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The configuration factory service.
   */
  public function __construct(ModuleHandlerInterface $module_handler, ConfigFactoryInterface $config_factory) {
    $this->moduleHandler = $module_handler;
    $this->configFactory = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('module_handler'),
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'jstimer_admin_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['jstimer.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->configFactory->get('jstimer.settings');

    // Check if the submodules are enabled.
    $submodules = ['jst_clock', 'jst_timer'];

    $sub_modules_enabled = FALSE;
    foreach ($submodules as $submodule) {
      if ($this->moduleHandler->moduleExists($submodule)) {
        $sub_modules_enabled = TRUE;
        break;
      }
    }

    // Display warning message with a link to the Extend page
    // if submodules are not enabled.
    if (!$sub_modules_enabled) {
      $extend_page_link = Link::fromTextAndUrl($this->t('Extend page'), Url::fromRoute('system.modules_list'))->toString();
      $this->messenger()->addWarning($this->t('Please enable at least one of the widget submodules (@submodules) to configure additional settings. Go to the @extend_page.', [
        '@extend_page' => $extend_page_link,
        '@submodules' => implode(', ', $submodules),
      ]));
    }

    // Placeholder method for global jstimer settings. Currently none.
    $form = parent::buildForm($form, $form_state);

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->configFactory->getEditable('jstimer.settings');

    $config->save();
    parent::submitForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    foreach ($form_state->getValues() as $key => $value) {
      if (substr($key, 0, 11) == "jstimer_js_") {
        if (!strpos($value, "'") === FALSE) {
          $form_state->setErrorByName($key, $this->t("Javascript Timer admin settings may not contain single quotes(')."));
        }
      }
    }
  }

}
